@@ -43,12 +43,15 @@ extern MODVAR struct stats *ircstp;
 extern MODVAR int bootopt;
 extern MODVAR time_t TSoffset;
 extern MODVAR time_t timeofday;
+extern MODVAR char cmodestring[512];
+extern MODVAR char umodestring[UMODETABLESZ+1];
 /* newconf */
 #define get_sendq(x) ((x)->local->class ? (x)->local->class->sendq : MAXSENDQLENGTH)
 /* get_recvq is only called in send.c for local connections */
 #define get_recvq(x) ((x)->local->class->recvq ? (x)->local->class->recvq : DEFAULT_RECVQ)
 
 #define CMD_FUNC(x) int (x) (aClient *cptr, aClient *sptr, int parc, char *parv[])
+#define CMD_OVERRIDE_FUNC(x) int (x)(Cmdoverride *ovr, aClient *cptr, aClient *sptr, int parc, char *parv[])
 
 /*
  * Configuration linked lists
@@ -236,6 +239,7 @@ extern void get_my_name(aClient *, char *, int);
 extern int get_sockerr(aClient *);
 extern int inetport(ConfigItem_listen *, char *, int, int);
 extern void init_sys();
+extern void check_user_limit(void);
 extern void init_modef();
 extern int verify_hostname(char *name);
 
@@ -294,7 +298,7 @@ extern void sendto_server(aClient *one, unsigned long caps, unsigned long nocaps
 extern void sendto_ops_and_log(char *pattern, ...) __attribute__((format(printf,1,2)));
 
 extern MODVAR int writecalls, writeb[];
-extern int deliver_it(aClient *, char *, int);
+extern int deliver_it(aClient *cptr, char *str, int len, int *want_read);
 extern int  check_for_target_limit(aClient *sptr, void *target, const char *name);
 extern char *canonize(char *buffer);
 extern ConfigItem_deny_dcc *dcc_isforbidden(aClient *sptr, char *filename);
@@ -432,17 +436,10 @@ extern int checkprotoflags(aClient *, int, char *, int);
 
 extern char *inetntop(int af, const void *in, char *local_dummy, size_t the_size);
 
-/*
- * CommandHash -Stskeeps
-*/
+/* Internal command stuff - not for modules */
 extern MODVAR aCommand *CommandHash[256];
-extern void	init_CommandHash(void);
-extern aCommand *add_Command_backend(char *cmd, int (*func)(), unsigned char parameters, int flags);
-extern void	add_Command(char *cmd, int (*func)(), unsigned char parameters);
-extern void	add_Command_to_list(aCommand *item, aCommand **list);
-extern aCommand *del_Command_from_list(aCommand *item, aCommand **list);
-extern int	del_Command(char *cmd, int (*func)());
-extern void    add_CommandX(char *cmd, int (*func)(), unsigned char parameters, int flags);
+extern void init_CommandHash(void);
+extern aCommand *add_Command_backend(char *cmd);
 
 /* CRULE */
 char *crule_parse(char *);
@@ -491,6 +488,7 @@ extern void      flag_add(char ch);
 extern void      flag_del(char ch);
 extern void init_dynconf(void);
 extern char *pretty_time_val(long);
+extern char *pretty_date(TS t);
 extern int        init_conf(char *filename, int rehash);
 extern void       validate_configuration(void);
 extern void       run_configuration(void);
@@ -556,7 +554,6 @@ extern void ExtbanDel(Extban *);
 extern void extban_init(void);
 extern char *trim_str(char *str, int len);
 extern MODVAR char *ban_realhost, *ban_virthost, *ban_ip;
-extern char *unreal_checkregex(char *s, int fastsupport, int check_broadness);
 extern int banact_stringtoval(char *s);
 extern char *banact_valtostring(int val);
 extern int banact_chartoval(char c);
@@ -669,6 +666,7 @@ extern MODVAR void (*send_join_to_local_users)(aClient *sptr, aChannel *chptr);
 extern MODVAR int (*do_nick_name)(char *nick);
 extern MODVAR int (*do_remote_nick_name)(char *nick);
 extern MODVAR char *(*charsys_get_current_languages)(void);
+extern MODVAR void *(*broadcast_sinfo)(aClient *acptr, aClient *to, aClient *except);
 /* /Efuncs */
 
 extern MODVAR aMotdFile opermotd, svsmotd, motd, botmotd, smotd, rules;
@@ -684,6 +682,7 @@ extern char *md5hash(unsigned char *dst, const unsigned char *src, unsigned long
 extern MODVAR char langsinuse[4096];
 extern MODVAR char *casemapping[2];
 extern MODVAR aTKline *tklines[TKLISTLEN];
+extern MODVAR aTKline *tklines_ip_hash[TKLIPHASHLEN1][TKLIPHASHLEN2];
 extern char *cmdname_by_spamftarget(int target);
 extern void unrealdns_delreq_bycptr(aClient *cptr);
 extern void sendtxtnumeric(aClient *to, char *pattern, ...) __attribute__((format(printf,2,3)));
@@ -740,6 +739,7 @@ extern MODVAR BOOL IsService;
 #endif
 extern int match_ip46(char *a, char *b);
 extern void extcmodes_check_for_changes(void);
+extern void umodes_check_for_changes(void);
 extern int config_parse_flood(char *orig, int *times, int *period);
 extern int swhois_add(aClient *acptr, char *tag, int priority, char *swhois, aClient *from, aClient *skip);
 extern int swhois_delete(aClient *acptr, char *tag, char *swhois, aClient *from, aClient *skip);
@@ -808,3 +808,13 @@ extern void setmaxtargets(char *cmd, int limit);
 extern void freemaxtargets(void);
 extern int max_targets_for_command(char *cmd);
 extern void set_targmax_defaults(void);
+extern void parse_chanmodes_protoctl(aClient *sptr, char *str);
+extern void concat_params(char *buf, int len, int parc, char *parv[]);
+extern void charsys_check_for_changes(void);
+extern int maxclients;
+extern int fast_badword_match(ConfigItem_badword *badword, char *line);
+extern int fast_badword_replace(ConfigItem_badword *badword, char *line, char *buf, int max);
+extern char *stripbadwords(char *str, ConfigItem_badword *start_bw, int *blocked);
+extern int badword_config_process(ConfigItem_badword *ca, char *str);
+extern void badword_config_free(ConfigItem_badword *ca);
+extern char *badword_config_check_regex(char *s, int fastsupport, int check_broadness);
