@@ -402,6 +402,18 @@ EVENT(garbage_collect)
 		loop.do_garbage_collect = 0;
 }
 
+EVENT(deprecated_notice)
+{
+	/* Send a warning to opers currently online every week after June 1, 2020 */
+	if (TStime() > 1590962400)
+	{
+		char *msg = "[WARNING] UnrealIRCd 4.x is no longer supported after December 31, 2020. "
+		            "See https://www.unrealircd.org/docs/UnrealIRCd_4_EOL";
+		sendto_realops("%s", msg);
+		ircd_log(LOG_ERROR, "%s", msg);
+	}
+}
+
 /*
 ** try_connections
 **
@@ -998,7 +1010,7 @@ int InitUnrealIRCd(int argc, char *argv[])
 	union pstun pstats;
 #endif
 	int  portarg = 0;
-#ifdef  FORCE_CORE
+#ifdef HAVE_SETRLIMIT
 	struct rlimit corelim;
 #endif
 
@@ -1074,10 +1086,10 @@ int InitUnrealIRCd(int argc, char *argv[])
 	extcmode_init();
 	init_random(); /* needs to be done very early!! */
 	clear_scache_hash_table();
-#ifdef FORCE_CORE
+#ifdef HAVE_SETRLIMIT
+	/* Make it so we can dump core */
 	corelim.rlim_cur = corelim.rlim_max = RLIM_INFINITY;
-	if (setrlimit(RLIMIT_CORE, &corelim))
-		printf("unlimit core size failed; errno = %d\n", errno);
+	setrlimit(RLIMIT_CORE, &corelim);
 #endif
 	/*
 	 * ** All command line parameters have the syntax "-fstring"
@@ -1306,12 +1318,19 @@ int InitUnrealIRCd(int argc, char *argv[])
 	fprintf(stderr, "%s", unreallogo);
 	fprintf(stderr, "                           v%s\n\n", VERSIONONLY);
 	fprintf(stderr, "  using %s\n", pcre2_version());
+#ifdef USE_TRE
 	fprintf(stderr, "  using %s\n", tre_version());
+#endif
 	fprintf(stderr, "  using %s\n", SSLeay_version(SSLEAY_VERSION));
 #ifdef USE_LIBCURL
 	fprintf(stderr, "  using %s\n", curl_version());
 #endif
+#endif
+	check_user_limit();
+#ifndef _WIN32
 	fprintf(stderr, "\n");
+	fprintf(stderr, "This server can handle %d concurrent sockets (%d clients + %d reserve)\n\n",
+		maxclients+CLIENTS_RESERVE, maxclients, CLIENTS_RESERVE);
 #endif
 	clear_client_hash_table();
 	clear_channel_hash_table();
@@ -1347,7 +1366,13 @@ int InitUnrealIRCd(int argc, char *argv[])
 	booted = TRUE;
 	load_tunefile();
 	make_umodestr();
+	me.flags = FLAGS_LISTEN;
+	me.fd = -1;
+	SetMe(&me);
+	make_server(&me);
 	extcmodes_check_for_changes();
+	umodes_check_for_changes();
+	charsys_check_for_changes();
 	clicap_init();
 	if (!find_Command_simple("AWAY") /*|| !find_Command_simple("KILL") ||
 		!find_Command_simple("OPER") || !find_Command_simple("PING")*/)
@@ -1374,15 +1399,17 @@ int InitUnrealIRCd(int argc, char *argv[])
 #ifndef _WIN32
 	fprintf(stderr, "Dynamic configuration initialized.. booting IRCd.\n");
 #endif
+	/* Warn about this starting March 1, 2020 */
+	if (time(NULL) > 1583017200)
+	{
+		fprintf(stderr, "WARNING: UnrealIRCd 4.x is no longer supported after December 31, 2020.\n"
+		                "See https://www.unrealircd.org/docs/UnrealIRCd_4_EOL\n");
+	}
 	open_debugfile();
 	if (portnum < 0)
 		portnum = PORTNUM;
 	me.local->port = portnum;
 	(void)init_sys();
-	me.flags = FLAGS_LISTEN;
-	me.fd = -1;
-	SetMe(&me);
-	make_server(&me);
 	applymeblock();
 #ifdef HAVE_SYSLOG
 	openlog("ircd", LOG_PID | LOG_NDELAY, LOG_DAEMON);
@@ -1409,7 +1436,9 @@ int InitUnrealIRCd(int argc, char *argv[])
 	me_hash = find_or_add(me.name);
 	me.serv->up = me_hash;
 	timeofday = time(NULL);
-	me.local->lasttime = me.local->since = me.local->firsttime = TStime();
+	me.local->lasttime = me.local->since = me.local->firsttime = me.serv->boottime = TStime();
+	me.serv->features.protocol = UnrealProtocol;
+	me.serv->features.software = strdup(version);
 	(void)add_to_client_hash_table(me.name, &me);
 	(void)add_to_id_hash_table(me.id, &me);
 	list_add(&me.client_node, &global_server_list);
