@@ -137,6 +137,7 @@ void (*send_join_to_local_users)(aClient *sptr, aChannel *chptr);
 int (*do_nick_name)(char *nick);
 int (*do_remote_nick_name)(char *nick);
 char *(*charsys_get_current_languages)(void);
+void *(*broadcast_sinfo)(aClient *acptr, aClient *to, aClient *except);
 
 static const EfunctionsList efunction_table[MAXEFUNCTIONS] = {
 /* 00 */	{NULL, NULL},
@@ -199,7 +200,8 @@ static const EfunctionsList efunction_table[MAXEFUNCTIONS] = {
 /* 57 */	{"do_nick_name", (void *)&do_nick_name},
 /* 58 */	{"do_remote_nick_name", (void *)&do_remote_nick_name},
 /* 59 */	{"charsys_get_current_languages", (void *)&charsys_get_current_languages},
-/* 60 */	{NULL, NULL}
+/* 60 */	{"broadcast_sinfo", (void *)&broadcast_sinfo},
+/* 61 */	{NULL, NULL}
 };
 
 #ifdef UNDERSCORE
@@ -1390,11 +1392,12 @@ Callback *CallbackDel(Callback *cb)
 
 Efunction	*EfunctionAddMain(Module *module, int eftype, int (*func)(), void (*vfunc)(), void *(*pvfunc)(), char *(*cfunc)())
 {
-Efunction *p;
+	Efunction *p;
 
 	if (!module || !(module->options & MOD_OPT_OFFICIAL))
 	{
-		module->errorcode = MODERR_INVALID;
+		if (module)
+			module->errorcode = MODERR_INVALID;
 		return NULL;
 	}
 	
@@ -1446,7 +1449,7 @@ Efunction *p, *q;
 	return NULL;
 }
 
-Cmdoverride *CmdoverrideAddEx(Module *module, char *name, int priority, iFP function)
+Cmdoverride *CmdoverrideAddEx(Module *module, char *name, int priority, OverrideCmdFunc function)
 {
 	aCommand *p;
 	Cmdoverride *ovr;
@@ -1491,7 +1494,7 @@ Cmdoverride *CmdoverrideAddEx(Module *module, char *name, int priority, iFP func
 	return ovr;
 }
 
-Cmdoverride *CmdoverrideAdd(Module *module, char *name, iFP function)
+Cmdoverride *CmdoverrideAdd(Module *module, char *name, OverrideCmdFunc function)
 {
 	return CmdoverrideAddEx(module, name, 0, function);
 }
