@@ -36,7 +36,7 @@ ModuleHeader MOD_HEADER(m_nick)
 /* Forward declarations */
 CMD_FUNC(m_nick);
 CMD_FUNC(m_uid);
-DLLFUNC int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, char *umode, char *virthost, char *ip);
+int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, char *umode, char *virthost, char *ip);
 int	AllowClient(aClient *cptr, struct hostent *hp, char *sockhost, char *username);
 int check_client(aClient *cptr, char *username);
 
@@ -243,6 +243,9 @@ void nick_collision(aClient *cptr, char *newnick, char *newid, aClient *new, aCl
 **	parv[10] = cloaked host, * if none
 **	parv[11] = ip
 **	parv[12] = info
+**
+** Technical documentation is available at:
+** https://www.unrealircd.org/docs/Server_protocol:UID_command
 */
 CMD_FUNC(m_uid)
 {
@@ -272,7 +275,7 @@ CMD_FUNC(m_uid)
 	 * For remote clients, do a quick check by using do_remote_nick_name(),
 	 * if this returned false then reject and kill it. -- Syzop
 	 */
-	if (IsServer(cptr) && !do_remote_nick_name(nick))
+	if (IsServer(cptr) && (!do_remote_nick_name(nick) || !*nick))
 	{
 		sendto_one(sptr, err_str(ERR_ERRONEUSNICKNAME),
 		    me.name, sptr->name, parv[1], "Illegal characters");
@@ -508,7 +511,8 @@ nickkill2done:
 		/* XXX: we need to split this out into register_remote_user() or something. */
 		parv[3] = nick;
 		parv[6] = sptr->name;
-		do_cmd(cptr, sptr, "USER", parc - 3, &parv[3]);
+		if (do_cmd(cptr, sptr, "USER", parc - 3, &parv[3]) == FLUSH_BUFFER)
+			return FLUSH_BUFFER;
 		if (GotNetInfo(cptr) && !IsULine(sptr))
 			sendto_fconnectnotice(sptr, 0, NULL);
 	}
@@ -589,7 +593,8 @@ CMD_FUNC(m_nick)
 	 * if this returned false then reject and kill it. -- Syzop
 	 */
 	if ((IsServer(cptr) && !do_remote_nick_name(nick)) ||
-	    (!IsServer(cptr) && !do_nick_name(nick)))
+	    (!IsServer(cptr) && !do_nick_name(nick)) ||
+	    !*nick)
 	{
 		sendto_one(sptr, err_str(ERR_ERRONEUSNICKNAME),
 		    me.name, sptr->name, parv[1], "Illegal characters");
@@ -1133,7 +1138,8 @@ CMD_FUNC(m_nick)
 	if (IsServer(cptr) && parc > 7)
 	{
 		parv[3] = nick;
-		do_cmd(cptr, sptr, "USER", parc - 3, &parv[3]);
+		if (do_cmd(cptr, sptr, "USER", parc - 3, &parv[3]) == FLUSH_BUFFER)
+			return FLUSH_BUFFER;
 		if (GotNetInfo(cptr) && !IsULine(sptr))
 			sendto_fconnectnotice(sptr, 0, NULL);
 	}
@@ -1173,8 +1179,6 @@ CMD_FUNC(m_nick)
 **	   this is not fair. It should actually request another
 **	   nick from local user or kill him/her...
 */
-extern MODVAR char cmodestring[512];
-extern MODVAR char umodestring[UMODETABLESZ+1];
 
 extern int short_motd(aClient *sptr);
 
@@ -1346,7 +1350,6 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 			RunHookReturnInt2(HOOKTYPE_REQUIRE_SASL, sptr, bconf->reason, !=0);
 			return banned_client(sptr, "Require-Auth", bconf->reason?bconf->reason:"", 0, 0);
 		}
-		tkl_check_expire(NULL);
 		/* Check G/Z lines before shuns -- kill before quite -- codemastr */
 		if ((xx = find_tkline_match(sptr, 0)) < 0)
 		{
@@ -1447,7 +1450,8 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 			char *parv[2];
 			parv[0] = sptr->name;
 			parv[1] = NULL;
-			do_cmd(sptr, sptr, "LUSERS", 1, parv);
+			if (do_cmd(sptr, sptr, "LUSERS", 1, parv) == FLUSH_BUFFER)
+				return FLUSH_BUFFER;
 		}
 
 		RunHook2(HOOKTYPE_WELCOME, sptr, 266);
@@ -1559,7 +1563,7 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 		 * if passwd given and if NickServ is online.
 		 * - by taz, modified by Wizzu
 		 */
-		if (sptr->local->passwd && SERVICES_NAME && (nsptr = find_person(NickServ, NULL)))
+		if (sptr->local->passwd && !IsLoggedIn(sptr) && SERVICES_NAME && (nsptr = find_person(NickServ, NULL)))
 		{
 			int do_identify = 1;
 			Hook *h;
@@ -1618,7 +1622,8 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 				tlds->channel,
 				NULL
 			};
-			do_cmd(sptr, sptr, "JOIN", 3, chans);
+			if (do_cmd(sptr, sptr, "JOIN", 3, chans) == FLUSH_BUFFER)
+				return FLUSH_BUFFER;
 		}
 		else if (!BadPtr(AUTO_JOIN_CHANS) && strcmp(AUTO_JOIN_CHANS, "0"))
 		{
@@ -1627,7 +1632,8 @@ int _register_user(aClient *cptr, aClient *sptr, char *nick, char *username, cha
 				AUTO_JOIN_CHANS,
 				NULL
 			};
-			do_cmd(sptr, sptr, "JOIN", 3, chans);
+			if (do_cmd(sptr, sptr, "JOIN", 3, chans) == FLUSH_BUFFER)
+				return FLUSH_BUFFER;
 		}
 		/* NOTE: If you add something here.. be sure to check the 'if (savetkl)' note above */
 	}
