@@ -22,6 +22,7 @@
 
 #include "unrealircd.h"
 
+/* Forward declarations */
 void send_channel_modes(aClient *cptr, aChannel *chptr);
 void send_channel_modes_sjoin(aClient *cptr, aChannel *chptr);
 void send_channel_modes_sjoin3(aClient *cptr, aChannel *chptr);
@@ -32,10 +33,11 @@ void _send_protoctl_servers(aClient *sptr, int response);
 void _send_server_message(aClient *sptr);
 void _introduce_user(aClient *to, aClient *acptr);
 int _check_deny_version(aClient *cptr, char *software, int protocol, char *flags);
+void _broadcast_sinfo(aClient *acptr, aClient *to, aClient *except);
 
+/* Global variables */
 static char buf[BUFSIZE];
 
-
 #define MSG_SERVER 	"SERVER"	
 
 ModuleHeader MOD_HEADER(m_server)
@@ -55,6 +57,7 @@ MOD_TEST(m_server)
 	EfunctionAdd(modinfo->handle, EFUNC_VERIFY_LINK, _verify_link);
 	EfunctionAddVoid(modinfo->handle, EFUNC_INTRODUCE_USER, _introduce_user);
 	EfunctionAdd(modinfo->handle, EFUNC_CHECK_DENY_VERSION, _check_deny_version);
+	EfunctionAddVoid(modinfo->handle, EFUNC_BROADCAST_SINFO, _broadcast_sinfo);
 	return MOD_SUCCESS;
 }
 
@@ -317,11 +320,42 @@ skip_host_check:
 	/* Now for checking passwords */
 	if (Auth_Check(cptr, link->auth, cptr->local->passwd) == -1)
 	{
+		/* Let's help admins a bit with a good error message in case
+		 * they mix different authentication systems (plaintext password
+		 * vs an "TLS Auth type" like spkifp/sslclientcert/sslclientcertfp).
+		 * The 'if' statement below is a bit complex but it consists of 2 things:
+		 * 1. Check if our side expects a plaintext password but we did not receive one
+		 * 2. Check if our side expects a non-plaintext password but we did receive one
+		 */
+		if (((link->auth->type == AUTHTYPE_PLAINTEXT) && cptr->local->passwd && !strcmp(cptr->local->passwd, "*")) ||
+		    ((link->auth->type != AUTHTYPE_PLAINTEXT) && cptr->local->passwd && strcmp(cptr->local->passwd, "*")))
+		{
+			sendto_ops_and_log("Link denied for '%s' (Authentication failed due to different password types on both sides of the link) %s",
+				servername, inpath);
+			sendto_ops_and_log("Read https://www.unrealircd.org/docs/FAQ#auth-fail-mixed for more information");
+		} else
+		if (link->auth->type == AUTHTYPE_SPKIFP)
+		{
+			sendto_ops_and_log("Link denied for '%s' (Authentication failed [spkifp mismatch]) %s",
+				servername, inpath);
+		} else
+		if (link->auth->type == AUTHTYPE_SSL_CLIENTCERT)
+		{
+			sendto_ops_and_log("Link denied for '%s' (Authentication failed [sslclientcert mismatch]) %s",
+				servername, inpath);
+		} else
+		if (link->auth->type == AUTHTYPE_SSL_CLIENTCERTFP)
+		{
+			sendto_ops_and_log("Link denied for '%s' (Authentication failed [sslclientcertfp mismatch]) %s",
+				servername, inpath);
+		} else
+		{
+			sendto_ops_and_log("Link denied for '%s' (Authentication failed [Bad password?]) %s",
+				servername, inpath);
+		}
 		sendto_one(cptr,
 		    "ERROR :Link '%s' denied (Authentication failed) %s",
 		    servername, inpath);
-		sendto_ops_and_log("Link denied for '%s' (Authentication failed [Bad password?]) %s",
-			servername, inpath);
 		return exit_client(cptr, sptr, &me,
 		    "Link denied (Authentication failed)");
 	}
@@ -815,6 +849,49 @@ void tls_link_notification_verify(aClient *acptr, ConfigItem_link *aconf)
 	}
 }
 
+#define SafeStr(x)    ((x && *(x)) ? (x) : "*")
+
+/** Broadcast SINFO.
+ * @param cptr   The server to send the information about.
+ * @param to     The server to send the information TO (NULL for broadcast).
+ * @param except The direction NOT to send to.
+ * This function takes into account that the server may not
+ * provide all of the detailed info. If any information is
+ * absent we will send 0 for numbers and * for NULL strings.
+ */
+void _broadcast_sinfo(aClient *acptr, aClient *to, aClient *except)
+{
+	char chanmodes[128], buf[512];
+
+	if (acptr->serv->features.chanmodes[0])
+	{
+		snprintf(chanmodes, sizeof(chanmodes), "%s,%s,%s,%s",
+			 acptr->serv->features.chanmodes[0],
+			 acptr->serv->features.chanmodes[1],
+			 acptr->serv->features.chanmodes[2],
+			 acptr->serv->features.chanmodes[3]);
+	} else {
+		strlcpy(chanmodes, "*", sizeof(chanmodes));
+	}
+
+	snprintf(buf, sizeof(buf), "%ld %d %s %s %s :%s",
+		      acptr->serv->boottime,
+		      acptr->serv->features.protocol,
+		      SafeStr(acptr->serv->features.usermodes),
+		      chanmodes,
+		      SafeStr(acptr->serv->features.nickchars),
+		      SafeStr(acptr->serv->features.software));
+
+	if (to)
+	{
+		/* Targetted to one server */
+		sendto_one(to, ":%s SINFO %s", acptr->name, buf);
+	} else {
+		/* Broadcast (except one side...) */
+		sendto_server(except, 0, 0, ":%s SINFO %s", acptr->name, buf);
+	}
+}
+
 int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 {
 	char		*inpath = get_client_name(cptr, TRUE);
@@ -912,6 +989,7 @@ int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 	cptr->local->class = cptr->serv->conf->class;
 	RunHook(HOOKTYPE_SERVER_CONNECT, cptr);
 
+	/* Broadcast new server to the rest of the network */
 	if (*cptr->id)
 	{
 		sendto_server(cptr, PROTO_SID, 0, ":%s SID %s 2 %s :%s",
@@ -922,7 +1000,12 @@ int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 		    cptr->serv->up,
 		    cptr->name, cptr->info);
 
-	send_moddata_client(cptr, &me); /* send moddata of &me (if any, likely minimal) */
+	/* Broadcast the just-linked-in featureset to other servers on our side */
+	broadcast_sinfo(cptr, NULL, cptr);
+
+	/* Send moddata of &me (if any, likely minimal) */
+	send_moddata_client(cptr, &me);
+
 	list_for_each_entry_reverse(acptr, &global_server_list, client_node)
 	{
 		/* acptr->from == acptr for acptr == cptr */
@@ -961,6 +1044,8 @@ int	m_server_synch(aClient *cptr, ConfigItem_link *aconf)
 					cptr->name, acptr->name);
 #endif
 			}
+			/* Send SINFO of our servers to their side */
+			broadcast_sinfo(acptr, cptr, NULL);
 			send_moddata_client(cptr, acptr); /* send moddata of server 'acptr' (if any, likely minimal) */
 		}
 	}
