@@ -1304,10 +1304,6 @@ int stats_set(aClient *sptr, char *para)
 			sptr->name, pretty_time_val(IDENT_CONNECT_TIMEOUT));
 	sendto_one(sptr, ":%s %i %s :ident::read-timeout: %s", me.name, RPL_TEXT,
 			sptr->name, pretty_time_val(IDENT_READ_TIMEOUT));
-	sendto_one(sptr, ":%s %i %s :modef-default-unsettime: %hd", me.name, RPL_TEXT,
-			sptr->name, (unsigned short)MODEF_DEFAULT_UNSETTIME);
-	sendto_one(sptr, ":%s %i %s :modef-max-unsettime: %hd", me.name, RPL_TEXT,
-			sptr->name, (unsigned short)MODEF_MAX_UNSETTIME);
 	sendto_one(sptr, ":%s %i %s :spamfilter::ban-time: %s", me.name, RPL_TEXT,
 		sptr->name, pretty_time_val(SPAMFILTER_BAN_TIME));
 	sendto_one(sptr, ":%s %i %s :spamfilter::ban-reason: %s", me.name, RPL_TEXT,
